@@ -3,16 +3,24 @@ from accessoryFunctions.accessoryFunctions import printtime, run_subprocess, wri
     combinetargets, MetadataObject, GenObject, make_dict
 from Bio.Blast.Applications import NcbiblastnCommandline
 from Bio.Application import ApplicationError
+from Bio.pairwise2 import format_alignment
+from Bio.SeqRecord import SeqRecord
 from Bio.Alphabet import IUPAC
+from Bio import pairwise2
 from Bio.Seq import Seq
 from Bio import SeqIO
 from collections import defaultdict
 from threading import Thread
 from csv import DictReader
+from queue import Queue
 from glob import glob
 import threading
+import xlsxwriter
 import time
+import csv
+import sys
 import os
+import re
 
 __author__ = 'adamkoziol'
 
@@ -95,14 +103,18 @@ class GeneSeekr(object):
                 pass
             # Find the best hit for each location based on percent identity
             for contig in resultdict:
+                # Do not allow the same gene to be added to the dictionary more than once
+                genes = list()
                 for location in resultdict[contig]:
                     # Initialise a variable to determine whether there is already a best hit found for the location
                     multiple = False
                     # Iterate through the BLAST results to find the best hit
                     for row in rowdict[contig][location]:
                         # Add the best hit to the .blastresults attribute of the object
-                        if row['percentidentity'] == max(resultdict[contig][location]) and not multiple:
+                        if row['percentidentity'] == max(resultdict[contig][location]) and not multiple \
+                                and row['subject_id'] not in genes:
                             sample[self.analysistype].blastresults.append(row)
+                            genes.append(row['subject_id'])
                             multiple = True
 
     def makedbthreads(self):
@@ -167,6 +179,9 @@ class GeneSeekr(object):
             threads.start()
         # Populate threads for each gene, genome combination
         for sample in self.metadata:
+            make_path(sample[self.analysistype].reportdir)
+            sample[self.analysistype].report = os.path.join(
+                sample[self.analysistype].reportdir, '{}.csv'.format(sample.name))
             if sample[self.analysistype].combinedtargets != 'NA':
                 # Add each fasta file combination to the threads
                 self.blastqueue.put((sample.general.bestassemblyfile, sample[self.analysistype].combinedtargets,
@@ -177,12 +192,8 @@ class GeneSeekr(object):
     def runblast(self):
         while True:  # while daemon
             (assembly, target, sample) = self.blastqueue.get()  # grabs fastapath from dqueue
-            genome = os.path.splitext(os.path.split(assembly)[1])[0]
             # Run the BioPython BLASTn module with the genome as query, fasta(target gene) as db.
             # Do not re-perform the BLAST search each time
-            make_path(sample[self.analysistype].reportdir)
-            sample[self.analysistype].report = os.path.join(
-                sample[self.analysistype].reportdir, '{}.csv'.format(genome))
             try:
                 size = os.path.getsize(sample[self.analysistype].report)
                 # If a report was created, but no results entered - program crashed, or no sequences passed thresholds,
@@ -198,8 +209,6 @@ class GeneSeekr(object):
             # alignments are reported. Also note the custom outfmt: the doubled quotes are necessary to get it work
             blastn = NcbiblastnCommandline(query=assembly, db=db, evalue='1E-5', num_alignments=1000000,
                                            num_threads=12,
-                                           # outfmt="'6 qseqid sseqid positive mismatch gaps "
-                                           #        "evalue bitscore slen length'",
                                            outfmt="'6 qseqid sseqid positive mismatch gaps "
                                                   "evalue bitscore slen length qstart qend qseq sstart send sseq'",
                                            out=sample[self.analysistype].report)
@@ -209,7 +218,7 @@ class GeneSeekr(object):
             if not os.path.isfile(sample[self.analysistype].report):
                 try:
                     blastn()
-                except:
+                except ApplicationError:
                     self.blastqueue.task_done()
                     self.blastqueue.join()
                     try:
@@ -270,6 +279,9 @@ class GeneSeekr(object):
         :param report: Name of the blast output report being parsed
         :param sample: sample object
         """
+        # Encountering the following error: # _csv.Error: field larger than field limit (131072)
+        # According to https://stackoverflow.com/a/15063941, increasing the field limit should fix the issue
+        csv.field_size_limit(sys.maxsize)
         # Open the sequence profile file as a dictionary
         blastdict = DictReader(open(report), fieldnames=self.fieldnames, dialect='excel-tab')
         # Initialise a dictionary to store all the target sequences
@@ -358,7 +370,6 @@ class GeneSeekr(object):
         """
         Creates .xlsx reports using xlsxwriter
         """
-        import xlsxwriter
         # Create a workbook to store the report. Using xlsxwriter rather than a simple csv format, as I want to be
         # able to have appropriately sized, multi-line cells
         workbook = xlsxwriter.Workbook(os.path.join(self.reportpath, '{}.xlsx'.format(self.analysistype)))
@@ -468,9 +479,6 @@ class GeneSeekr(object):
         """
         Create alignments of the sample nucleotide and amino acid sequences to the reference sequences
         """
-        import re
-        from Bio import pairwise2
-        from Bio.pairwise2 import format_alignment
         # Initialise dictionaries
         sample[self.analysistype].dnaseq = dict()
         sample[self.analysistype].protseq = dict()
@@ -550,26 +558,15 @@ class GeneSeekr(object):
         """
         Custom reports for ResFinder analyses. These reports link the gene(s) found to their resistance phenotypes
         """
-        import xlsxwriter
-        from Bio.SeqRecord import SeqRecord
-        for sample in self.metadata:
-            if sample.general.bestassemblyfile != 'NA':
-                # Create a dictionary to store the genotype: phenotype data
-                sample[self.analysistype].resfindernotes = dict()
-                with open(os.path.join(sample[self.analysistype].targetpath, 'notes.txt')) as notes:
-                    for line in notes:
-                        # Ignore lines starting with a '#' these lines usually denote classes of antibiotics
-                        if not line.startswith('#'):
-                            # Example line: aac(2')-Ia:Aminoglycoside resistance:
-                            # Gene: aac(2')-Ia
-                            gene = line.split(':')[0]
-                            # Resistance: Aminoglycoside resistance
-                            resistance = line.split(':')[1]
-                            sample[self.analysistype].resfindernotes[gene] = resistance
 
+        from spadespipeline.typingclasses import ResistanceNotes
+        target_dir = str()
+        for folder in self.targetfolders:
+            target_dir = folder
+        genedict, altgenedict = ResistanceNotes.notes(target_dir)
         # Create a workbook to store the report. Using xlsxwriter rather than a simple csv format, as I want to be
         # able to have appropriately sized, multi-line cells
-        workbook = xlsxwriter.Workbook('{}/{}.xlsx'.format(self.reportpath, self.analysistype))
+        workbook = xlsxwriter.Workbook(os.path.join(self.reportpath, '{}.xlsx'.format(self.analysistype)))
         # New worksheet to store the data
         worksheet = workbook.add_worksheet()
         # Add a bold format for header cells. Using a monotype font size 10
@@ -583,94 +580,75 @@ class GeneSeekr(object):
         # A dictionary to store the column widths for every header
         columnwidth = dict()
         extended = False
-        headers = ['Strain', 'Gene', 'Resistance', 'PercentIdentity', 'PercentCovered', 'Contig', 'Location',
+        headers = ['Strain', 'Gene', 'Allele', 'Resistance', 'PercentIdentity', 'PercentCovered', 'Contig', 'Location',
                    'nt_sequence']
         for sample in self.metadata:
             sample[self.analysistype].sampledata = list()
-            # Initialise a list of all the headers with 'Strain'
-            if sample[self.analysistype].targetnames != 'NA':
-                # Process the sample only if the script could find targets
-                if sample[self.analysistype].blastresults != 'NA':
-                    for result in sample[self.analysistype].blastresults:
-                        # Initialise a list to store all the data for each strain
-                        data = list()
-                        # There are three 'styles' of gene:resistance pairs in the notes.txt file
-                        # 1) strA:Aminoglycoside resistance, with an allele format: strA_1_M96392
-                        # Only the gene name (strA) is required
-                        try:
-                            target = result['subject_id'].split('_')[0]
-                            resistance = sample[self.analysistype].resfindernotes[target]
-                        # 2) blaTEM-101:Beta-lactam resistance, with an allele format: blaTEM_1B_1_JF910132
-                        # The '1B' following the gene name (blaTEM) is also required
-                        except KeyError:
-                            try:
-                                target = '-'.join([result['subject_id'].split('_')[0],
-                                                   result['subject_id'].split('_')[1]])
-                                resistance = sample[self.analysistype].resfindernotes[target]
-                            # 3) blaCTX-M-55:Beta-lactam resistance, with an allele format: blaCTX_M_55_2_GQ456159
-                            # The 'M' and the '55' following the gene name (blaCTX) are both required
-                            except KeyError:
-                                target = '-'.join([result['subject_id'].split('_')[0],
-                                                   result['subject_id'].split('_')[1],
-                                                   result['subject_id'].split('_')[2]])
-                                resistance = sample[self.analysistype].resfindernotes[target]
+            # Process the sample only if the script could find targets
+            if sample[self.analysistype].blastresults != 'NA' and sample[self.analysistype].blastresults:
+                for result in sample[self.analysistype].blastresults:
+                    # Set the name to avoid writing out the dictionary[key] multiple times
+                    name = result['subject_id']
+                    # Use the ResistanceNotes gene name extraction method to get the necessary variables
+                    gname, genename, accession, allele = ResistanceNotes.gene_name(name)
+                    # Initialise a list to store all the data for each strain
+                    data = list()
+                    # Determine the name of the gene to use in the report and the resistance using the resistance
+                    # method
+                    finalgene, resistance = ResistanceNotes.resistance(gname, genename, genedict, altgenedict)
+                    # Append the necessary values to the data list
+                    data.append(finalgene)
+                    data.append(allele)
+                    data.append(resistance)
+                    percentid = result['percentidentity']
+                    data.append(percentid)
+                    data.append(result['alignment_fraction'])
+                    data.append(result['query_id'])
+                    data.append('...'.join([str(result['low']), str(result['high'])]))
+                    try:
+                        # Only if the alignment option is selected, for inexact results, add alignments
+                        if self.align and percentid != 100.00:
 
-                        # Append the necessary values to the data list
-                        data.append(result['subject_id'])
-                        data.append(resistance)
-                        percentid = result['percentidentity']
-                        data.append(percentid)
-                        data.append(result['alignment_fraction'])
-                        data.append(result['query_id'])
-                        data.append('...'.join([str(result['low']), str(result['high'])]))
-                        try:
-                            # Only if the alignment option is selected, for inexact results, add alignments
-                            if self.align and percentid != 100.00:
-
-                                # Align the protein (and nucleotide) sequences to the reference
-                                self.alignprotein(sample, result['subject_id'])
-                                if not extended:
-                                    # Add the appropriate headers
-                                    headers.extend(['aa_Identity',
-                                                    'aa_Alignment',
-                                                    'aa_SNP_location',
-                                                    'nt_Alignment',
-                                                    'nt_SNP_location'
-                                                    ])
-                                    extended = True
-                                # Create a FASTA-formatted sequence output of the query sequence
-                                record = SeqRecord(sample[self.analysistype].dnaseq[result['subject_id']],
-                                                   id='{}_{}'.format(sample.name, result['subject_id']),
-                                                   description='')
-
-                                # Add the alignment, and the location of mismatches for both nucleotide and amino
-                                # acid sequences
-                                data.extend([record.format('fasta'),
-                                             sample[self.analysistype].aaidentity[result['subject_id']],
-                                             sample[self.analysistype].aaalign[result['subject_id']],
-                                             sample[self.analysistype].aaindex[result['subject_id']],
-                                             sample[self.analysistype].ntalign[result['subject_id']],
-                                             sample[self.analysistype].ntindex[result['subject_id']]
-                                             ])
-                            else:
-                                record = SeqRecord(Seq(result['subject_sequence'], IUPAC.unambiguous_dna),
-                                                   id='{}_{}'.format(sample.name, result['subject_id']),
-                                                   description='')
-                                data.append(record.format('fasta'))
-                                if self.align:
-                                    # Add '-'s for the empty results, as there are no alignments for exact matches
-                                    data.extend(['-', '-', '-', '-', '-'])
-                        # If there are no blast results for the target, add a '-'
-                        except (KeyError, TypeError):
-                            data.append('-')
-                        sample[self.analysistype].sampledata.append(data)
-                # If there are no blast results at all, add a '-'
-                # else:
-                #     data.append('-')
+                            # Align the protein (and nucleotide) sequences to the reference
+                            self.alignprotein(sample, name)
+                            if not extended:
+                                # Add the appropriate headers
+                                headers.extend(['aa_Identity',
+                                                'aa_Alignment',
+                                                'aa_SNP_location',
+                                                'nt_Alignment',
+                                                'nt_SNP_location'
+                                                ])
+                                extended = True
+                            # Create a FASTA-formatted sequence output of the query sequence
+                            record = SeqRecord(sample[self.analysistype].dnaseq[name],
+                                               id='{}_{}'.format(sample.name, name),
+                                               description='')
 
+                            # Add the alignment, and the location of mismatches for both nucleotide and amino
+                            # acid sequences
+                            data.extend([record.format('fasta'),
+                                         sample[self.analysistype].aaidentity[name],
+                                         sample[self.analysistype].aaalign[name],
+                                         sample[self.analysistype].aaindex[name],
+                                         sample[self.analysistype].ntalign[name],
+                                         sample[self.analysistype].ntindex[name]
+                                         ])
+                        else:
+                            record = SeqRecord(Seq(result['query_sequence'], IUPAC.unambiguous_dna),
+                                               id='{}_{}'.format(sample.name, name),
+                                               description='')
+                            data.append(record.format('fasta'))
+                            if self.align:
+                                # Add '-'s for the empty results, as there are no alignments for exact matches
+                                data.extend(['-', '-', '-', '-', '-'])
+                    # If there are no blast results for the target, add a '-'
+                    except (KeyError, TypeError):
+                        data.append('-')
+                    sample[self.analysistype].sampledata.append(data)
         if 'nt_sequence' not in headers:
             headers.append('nt_sequence')
-        # Write the header to the spreadsheet
+            # Write the header to the spreadsheet
         for header in headers:
             worksheet.write(row, col, header, bold)
             # Set the column width based on the longest header
@@ -681,19 +659,24 @@ class GeneSeekr(object):
                 columnwidth[col] = len(header)
             worksheet.set_column(col, col, columnwidth[col])
             col += 1
-        # Increment the row and reset the column to zero in preparation of writing results
+            # Increment the row and reset the column to zero in preparation of writing results
         row += 1
         col = 0
         # Write out the data to the spreadsheet
         for sample in self.metadata:
-            worksheet.write(row, col, sample.name, courier)
-            columnwidth[col] = len(sample.name)
-            worksheet.set_column(col, col, columnwidth[col])
-            col += 1
-            multiple = False
+            if not sample[self.analysistype].sampledata:
+                worksheet.write(row, col, sample.name, courier)
+                # Increment the row and reset the column to zero in preparation of writing results
+                row += 1
+                col = 0
+                # Set the width of the row to be the number of lines (number of newline characters) * 12
+                worksheet.set_row(row)
+                worksheet.set_column(col, col, columnwidth[col])
             for data in sample[self.analysistype].sampledata:
-                if multiple:
-                    col += 1
+                columnwidth[col] = len(sample.name) + 2
+                worksheet.set_column(col, col, columnwidth[col])
+                worksheet.write(row, col, sample.name, courier)
+                col += 1
                 # List of the number of lines for each result
                 totallines = list()
                 for results in data:
@@ -730,7 +713,6 @@ class GeneSeekr(object):
                         columnwidth[col] = alignmentcorrect
                     worksheet.set_column(col, col, columnwidth[col])
                     col += 1
-                    multiple = True
                 # Set the width of the row to be the number of lines (number of newline characters) * 12
                 worksheet.set_row(row, max(totallines) * 11)
                 # Increase the row counter for the next strain's data
@@ -809,7 +791,7 @@ class GeneSeekr(object):
         return blaststring
 
     def __init__(self, inputobject):
-        from queue import Queue
+
         self.metadata = inputobject.runmetadata.samples
         self.cutoff = inputobject.cutoff
         self.start = inputobject.start
@@ -841,7 +823,6 @@ class GeneSeekr(object):
         self.plusdict = defaultdict(make_dict)
         self.dqueue = Queue(maxsize=self.cpus)
         self.blastqueue = Queue(maxsize=self.cpus)
-        self.geneseekr()
 
 
 def sequencenames(contigsfile):
@@ -863,7 +844,7 @@ if __name__ == '__main__':
         def strainer(self):
             # Get the sequences in the sequences folder into a list. Note that they must have a file extension that
             # begins with .fa
-            self.strains = sorted(glob('{}*.fa*'.format(self.sequencepath)))
+            self.strains = sorted(glob(os.path.join(self.sequencepath, '*.fa*'.format(self.sequencepath))))
             self.targets = sorted(glob(os.path.join(self.targetpath, '*.tfa')))
             try:
                 self.combinedtargets = glob(os.path.join(self.targetpath, '*.fasta'))[0]
@@ -935,13 +916,13 @@ if __name__ == '__main__':
                                 action='store_true',
                                 help='Perform VirulenceFinder-like analyses')
             args = parser.parse_args()
-            self.sequencepath = os.path.join(args.sequencepath, '')
+            self.sequencepath = os.path.join(args.sequencepath)
             assert os.path.isdir(self.sequencepath), 'Cannot locate sequence path as specified: {}'\
                 .format(self.sequencepath)
-            self.targetpath = os.path.join(args.targetpath, '')
+            self.targetpath = os.path.join(args.targetpath)
             assert os.path.isdir(self.targetpath), 'Cannot locate target path as specified: {}'\
                 .format(self.targetpath)
-            self.reportpath = os.path.join(args.reportpath, '')
+            self.reportpath = os.path.join(args.reportpath)
             make_path(self.reportpath)
             assert os.path.isdir(self.reportpath), 'Cannot locate report path as specified: {}'\
                 .format(self.reportpath)
@@ -961,8 +942,8 @@ if __name__ == '__main__':
             elif self.virulencefinder:
                 self.analysistype = 'virulence'
             elif self.resfinder and self.virulencefinder:
-                print('Cannot perform ResFinder and VirulenceFinder simultaneously. Please choose only one'
-                      'of the -R and -v flags')
+                printtime('Cannot perform ResFinder and VirulenceFinder simultaneously. Please choose only one '
+                          'of the -R and -v flags', self.start)
             else:
                 self.analysistype = 'geneseekr'
             self.start = time.time()
@@ -984,7 +965,8 @@ if __name__ == '__main__':
             self.unique = self.runmetadata.unique
             self.logfile = self.runmetadata.logfile
             # Run the analyses
-            GeneSeekr(self)
+            geneseekr = GeneSeekr(self)
+            geneseekr.geneseekr()
 
     # Run the class
     MetadataInit()
@@ -996,9 +978,12 @@ class PipelineInit(object):
             if sample.general.bestassemblyfile != 'NA':
                 setattr(sample, self.analysistype, GenObject())
                 if self.genusspecific:
+                    try:
+                        genus = sample.general.closestrefseqgenus
+                    except AttributeError:
+                        genus = sample.general.referencegenus
                     # Allow Shigella to use the same targets as Escherichia
-                    genus = sample.general.referencegenus if sample.general.referencegenus != 'Shigella' \
-                        else 'Escherichia'
+                    genus = genus if genus != 'Shigella' else 'Escherichia'
                     targetpath = os.path.join(self.referencefilepath, self.analysistype, genus)
                 else:
                     targetpath = os.path.join(self.referencefilepath, self.analysistype)
