@@ -1,19 +1,20 @@
 #!/usr/bin/env python3
-from glob import glob
-from threading import Lock, Thread
-import threading
-import re
-from Bio.Sequencing.Applications import SamtoolsViewCommandline, SamtoolsSortCommandline, SamtoolsIndexCommandline
 from accessoryFunctions.accessoryFunctions import make_path, run_subprocess, write_to_logfile, logstr, GenObject, \
-    printtime, get_version, MetadataObject
+    get_version, MetadataObject
 from spadespipeline.bowtie import Bowtie2BuildCommandLine, Bowtie2CommandLine
+from Bio.Sequencing.Applications import SamtoolsViewCommandline, SamtoolsSortCommandline, SamtoolsIndexCommandline
 from Bio.Application import ApplicationError
-from io import StringIO
-import os
 from Bio import SeqIO
+from threading import Lock, Thread
+from click import progressbar
+from io import StringIO
+from glob import glob
+import threading
+import logging
 import shutil
+import os
+import re
 
-threadlock = Lock()
 __author__ = 'mike knowles, adamkoziol'
 
 
@@ -23,10 +24,10 @@ class QualiMap(object):
         """
         Run the methods in the correct order
         """
-        printtime('Aligning reads with bowtie2 for Qualimap', self.start)
+        logging.info('Aligning reads with bowtie2 for Qualimap')
         self.bowtie()
         self.indexing()
-        # self.pilon()
+        self.pilon()
         self.filter()
         self.clear()
 
@@ -41,58 +42,59 @@ class QualiMap(object):
             threads.setDaemon(True)
             # Start the threading
             threads.start()
-        for sample in self.metadata:
-            # Initialise the mapping GenObject
-            sample.mapping = GenObject()
-            # Set an easier to write shortcut for sample.general
-            sagen = sample.general
-            if sagen.bestassemblyfile != "NA":
-                sagen.QualimapResults = os.path.join(sagen.outputdirectory, 'qualimap_results')
-                # Set the results folder
-                # Create this results folder if necessary
-                make_path(sagen.QualimapResults)
-                # Set file names
-                sagen.sortedbam = os.path.join(sagen.QualimapResults, '{}_sorted.bam'.format(sample.name))
-                filenoext = os.path.splitext(sagen.filteredfile)[0]
-                sagen.filenoext = filenoext
-                sagen.bowtie2results = os.path.join(sagen.QualimapResults, sample.name)
-                # Use fancy new bowtie2 wrapper
-                bowtie2build = Bowtie2BuildCommandLine(reference=sagen.bestassemblyfile,
-                                                       bt2=sagen.bowtie2results)
-                sample.mapping.BamFile = sagen.bowtie2results + "_sorted.bam"
-                # SAMtools sort v1.3 has different run parameters
-                if self.samversion < "1.3":
-                    samsort = SamtoolsSortCommandline(input="-", out_prefix=sample.mapping.BamFile)
-                else:
+        with progressbar(self.metadata) as bar:
+            for sample in bar:
+                # Initialise the mapping GenObject
+                sample.mapping = GenObject()
+                # Set an easier to write shortcut for sample.general
+                sagen = sample.general
+                if sagen.bestassemblyfile != "NA":
+                    sagen.QualimapResults = os.path.join(sagen.outputdirectory, 'qualimap_results')
+                    # Set the results folder
+                    # Create this results folder if necessary
+                    make_path(sagen.QualimapResults)
+                    # Set file names
+                    sagen.sortedbam = os.path.join(sagen.QualimapResults, '{}_sorted.bam'.format(sample.name))
+                    filenoext = os.path.splitext(sagen.filteredfile)[0]
+                    sagen.filenoext = filenoext
+                    sagen.bowtie2results = os.path.join(sagen.QualimapResults, sample.name)
+                    # Use fancy new bowtie2 wrapper
+                    bowtie2build = Bowtie2BuildCommandLine(reference=sagen.bestassemblyfile,
+                                                           bt2=sagen.bowtie2results)
+                    sample.mapping.BamFile = sagen.bowtie2results + "_sorted.bam"
+                    # SAMtools sort v1.3 has different run parameters
                     samsort = SamtoolsSortCommandline(input=sample.mapping.BamFile,
                                                       o=True,
                                                       out_prefix="-")
-                samtools = [SamtoolsViewCommandline(b=True, S=True, input_file="-"), samsort]
-                indict = {'D': 5, 'R': 1, 'num_mismatches': 0, 'seed_length': 22, 'i_func': "S,0,2.50"}
-                #  Update the dictionary with the appropriate parameters for paired- vs. single-ended assemblies
-                try:
-                    _ = sample.general.mergedreads
-                    if len(sample.general.trimmedcorrectedfastqfiles) == 2:
-                        indict.update({'m1': sample.general.trimmedcorrectedfastqfiles[0],
-                                       'm2': sample.general.trimmedcorrectedfastqfiles[1]})
-                    else:
-                        indict.update({'U': sample.general.trimmedcorrectedfastqfiles[0]})
-                except AttributeError:
-                    if len(sample.general.assemblyfastq) == 2:
-                        indict.update({'m1': sample.general.assemblyfastq[0], 'm2': sample.general.assemblyfastq[1]})
-                    else:
-                        indict.update({'U': sample.general.assemblyfastq[0]})
-                bowtie2align = Bowtie2CommandLine(bt2=sagen.bowtie2results,
-                                                  threads=self.threads,
-                                                  samtools=samtools,
-                                                  **indict)
+                    samtools = [SamtoolsViewCommandline(b=True, S=True, input_file="-"), samsort]
+                    indict = {'D': 5, 'R': 1, 'num_mismatches': 0, 'seed_length': 22, 'i_func': "S,0,2.50"}
+                    #  Update the dictionary with the appropriate parameters for paired- vs. single-ended assemblies
+                    try:
+                        _ = sample.general.mergedreads
+                        if len(sample.general.trimmedcorrectedfastqfiles) == 2:
+                            indict.update({'m1': sample.general.trimmedcorrectedfastqfiles[0],
+                                           'm2': sample.general.trimmedcorrectedfastqfiles[1]})
+                        else:
+                            indict.update({'U': sample.general.trimmedcorrectedfastqfiles[0]})
+                    except AttributeError:
+                        if len(sample.general.assemblyfastq) == 2:
+                            indict.update({'m1': sample.general.assemblyfastq[0],
+                                           'm2': sample.general.assemblyfastq[1]})
+                        else:
+                            indict.update({'U': sample.general.assemblyfastq[0]})
+                    bowtie2align = Bowtie2CommandLine(bt2=sagen.bowtie2results,
+                                                      threads=self.threads,
+                                                      samtools=samtools,
+                                                      **indict)
 
-                # Convert the commands to strings to allow them to be JSON serialized
-                sample.commands.bowtie2align = str(bowtie2align)
-                sample.commands.bowtie2build = str(bowtie2build)
-                self.bowqueue.put((sample, sample.commands.bowtie2build, sample.commands.bowtie2align))
-            else:
-                sample.commands.samtools = "NA"
+                    # Convert the commands to strings to allow them to be JSON serialized
+                    sample.commands.bowtie2align = str(bowtie2align)
+                    sample.commands.bowtie2build = str(bowtie2build)
+                    self.bowqueue.put((sample, sample.commands.bowtie2build, sample.commands.bowtie2align))
+                else:
+                    sample.commands.samtools = "NA"
+                    sample.mapping.MeanInsertSize = 'NA'
+                    sample.mapping.MeanCoveragedata = 'NA'
         self.bowqueue.join()
 
     def align(self):
@@ -100,29 +102,29 @@ class QualiMap(object):
         while True:
             sample, bowtie2build, bowtie2align = self.bowqueue.get()
             if sample.general.bestassemblyfile != 'NA':
-                if not os.path.isfile(sample.mapping.BamFile) and not os.path.isfile(sample.mapping.BamFile + ".bz2"):
+                if not os.path.isfile(sample.mapping.BamFile):
                     stdout = StringIO()
                     for func in bowtie2build, bowtie2align:
                         stdout.close()
                         out, err = run_subprocess(str(func))
-                        threadlock.acquire()
+                        self.threadlock.acquire()
                         write_to_logfile(str(func), str(func), self.logfile, sample.general.logout,
                                          sample.general.logerr, None, None)
                         write_to_logfile(out, err, self.logfile, sample.general.logout, sample.general.logerr,
                                          None, None)
-                        threadlock.release()
+                        self.threadlock.release()
             # For different alignment
             sam = sample.general.bowtie2results + ".sam"
             if os.path.isfile(sam):
                 # PIPE stdout to stdin of samtools view then sort (only outputting sorted bam)
-                # SAMtools sort v1.3 has different run parameters
-                if self.samversion < "1.3":
-                    samsort = SamtoolsSortCommandline(input="-", out_prefix=sample.mapping.BamFile[:-4])
-                else:
-                    samsort = SamtoolsSortCommandline(input=sample.mapping.BamFile, o=True, out_prefix="-")
+                samsort = SamtoolsSortCommandline(input=sample.mapping.BamFile, o=True, out_prefix="-")
                 # Use cStringIO streams to handle bowtie output
                 stdout = StringIO()
-                for func in [SamtoolsViewCommandline(b=True, S=True, input_file=sample.mapping.BamFile), samsort]:
+                for func in [SamtoolsViewCommandline(b=True,
+                                                     S=True,
+                                                     F=4,
+                                                     h=True,
+                                                     input_file=sample.mapping.BamFile), samsort]:
                     # Use closing contextmanager for handle __exit__() as close()
                     stdout, stderr = map(StringIO, func(stdin=stdout.getvalue()))
                     # Write the standard error to log
@@ -183,19 +185,17 @@ class QualiMap(object):
                                 except ValueError:
                                     pass
 
-            except IOError:
-                self.bowqueue.task_done()
-                self.bowqueue.join()
-                raise
+            except (IOError, FileNotFoundError):
+                pass
             # If there are values in the dictionary
             if qdict:
                 # Make new category for Qualimap results and populate this category with the report data
                 for attribute in qdict:
-                    # setattr(sample, "mapping", GenObject(qdict))
-                    setattr(sample.mapping, attribute, qdict[attribute])
+                    # Remove the 'X' from the depth values e.g. 40.238X
+                    setattr(sample.mapping, attribute, qdict[attribute].rstrip('X'))
 
     def indexing(self):
-        printtime('Indexing sorted bam files', self.start)
+        logging.info('Indexing sorted bam files')
         for i in range(self.cpus):
             # Send the threads to
             threads = Thread(target=self.index, args=())
@@ -203,12 +203,13 @@ class QualiMap(object):
             threads.setDaemon(True)
             # Start the threading
             threads.start()
-        for sample in self.metadata:
-            if sample.general.bestassemblyfile != 'NA':
-                bamindex = SamtoolsIndexCommandline(input=sample.mapping.BamFile)
-                sample.mapping.sortedbai = sample.mapping.BamFile + '.bai'
-                sample.mapping.bamindex = str(bamindex)
-                self.indexqueue.put((sample, bamindex))
+        with progressbar(self.metadata) as bar:
+            for sample in bar:
+                if sample.general.bestassemblyfile != 'NA':
+                    bamindex = SamtoolsIndexCommandline(input=sample.mapping.BamFile)
+                    sample.mapping.sortedbai = sample.mapping.BamFile + '.bai'
+                    sample.mapping.bamindex = str(bamindex)
+                    self.indexqueue.put((sample, bamindex))
         self.indexqueue.join()
 
     def index(self):
@@ -233,7 +234,7 @@ class QualiMap(object):
         """
         Run pilon to fix any misassemblies in the contigs - will look for SNPs and indels
         """
-        printtime('Improving quality of assembly with pilon', self.start)
+        logging.info('Improving quality of assembly with pilon')
         for i in range(self.cpus):
             # Send the threads to the merge method. :args is empty as I'm using
             threads = Thread(target=self.pilonthreads, args=())
@@ -241,20 +242,24 @@ class QualiMap(object):
             threads.setDaemon(True)
             # Start the threading
             threads.start()
-        for sample in self.metadata:
-            # Set the name of the unfiltered spades assembly output file
-            sample.general.contigsfile = os.path.join(sample.general.spadesoutput, 'contigs.fasta')
-            sample.mapping.pilondir = os.path.join(sample.general.QualimapResults, 'pilon')
-            make_path(sample.mapping.pilondir)
-            # I was getting a java.lang.OutOfMemoryError: GC overhead limit exceeded error, which I fixed by
-            # providing the -Xmx20G argument.
-            sample.mapping.piloncmd = 'pilon -Xmx40G --genome {} --bam {} --fix bases --threads {} ' \
-                                      '--outdir {} --changes --mindepth 0.25' \
-                .format(sample.general.contigsfile,
-                        sample.mapping.BamFile,
-                        self.threads,
-                        sample.mapping.pilondir)
-            self.pilonqueue.put(sample)
+        with progressbar(self.metadata) as bar:
+            for sample in bar:
+                if sample.general.bestassemblyfile != 'NA':
+                    if sample.general.polish:
+                        # Set the name of the unfiltered assembly output file
+                        sample.general.contigsfile = sample.general.assemblyfile
+                        sample.mapping.pilondir = os.path.join(sample.general.QualimapResults, 'pilon')
+                        make_path(sample.mapping.pilondir)
+                        # Create the command line command
+                        sample.mapping.piloncmd = 'pilon --genome {} --bam {} --fix bases --threads {} ' \
+                                                  '--outdir {} --changes --mindepth 0.25' \
+                            .format(sample.general.contigsfile,
+                                    sample.mapping.BamFile,
+                                    self.threads,
+                                    sample.mapping.pilondir)
+                        self.pilonqueue.put(sample)
+                    else:
+                        sample.general.contigsfile = sample.general.assemblyfile
         self.pilonqueue.join()
 
     def pilonthreads(self):
@@ -265,11 +270,11 @@ class QualiMap(object):
             if not os.path.isfile(sample.general.contigsfile):
                 command = sample.mapping.piloncmd
                 out, err = run_subprocess(command)
-                threadlock.acquire()
+                self.threadlock.acquire()
                 write_to_logfile(command, command, self.logfile, sample.general.logout, sample.general.logerr, None,
                                  None)
                 write_to_logfile(out, err, self.logfile, sample.general.logout, sample.general.logerr, None, None)
-                threadlock.release()
+                self.threadlock.release()
             self.pilonqueue.task_done()
 
     def filter(self):
@@ -277,24 +282,26 @@ class QualiMap(object):
         Filter contigs based on depth
         """
 
-        printtime('Filtering contigs', self.start)
+        logging.info('Filtering contigs')
         for i in range(self.cpus):
-            # Send the threads to the merge method. :args is empty as I'm using
+            # Send the threads to the filter method
             threads = Thread(target=self.filterthreads, args=())
             # Set the daemon to true - something to do with thread management
             threads.setDaemon(True)
             # Start the threading
             threads.start()
-        for sample in self.metadata:
-            # Set the name of the unfiltered spades assembly output file
-            sample.general.contigsfile = os.path.join(sample.general.spadesoutput, 'contigs.fasta')
-            self.filterqueue.put(sample)
+        with progressbar(self.metadata) as bar:
+            for sample in bar:
+                # Set the name of the unfiltered assembly output file
+                if sample.general.bestassemblyfile != 'NA':
+                    sample.general.contigsfile = sample.general.assemblyfile
+                    self.filterqueue.put(sample)
         self.filterqueue.join()
 
     def filterthreads(self):
         while True:
             sample = self.filterqueue.get()
-            # Only run on samples that have been processed with spades
+            # Only run on samples that have been assembled
             if os.path.isfile(sample.general.contigsfile) and not os.path.isfile(sample.general.filteredfile):
                 # Create a list to store all the records of contigs that pass the minimum depth filtering
                 passdepth = list()
@@ -311,8 +318,8 @@ class QualiMap(object):
                             and len(record.seq) > 500:
                         # Replace 'NODE' in the fasta header with the sample name
                         # >NODE_1_length_705814_cov_37.107_ID_4231
-                        newid = re.sub("NODE", sample.name, record.id)
-                        record.id = str(record.id).replace('NODE', sample.name)
+                        newid = re.sub("Contig", sample.name, record.id)
+                        record.id = str(record.id).replace('Contig', sample.name)
                         record.id = newid
                         # Clear the name and description attributes of the record
                         record.name = ''
@@ -329,8 +336,6 @@ class QualiMap(object):
             if os.path.isfile(sample.general.filteredfile):
                 # Set the assemblies path
                 sample.general.bestassembliespath = os.path.join(self.path, 'BestAssemblies')
-                # Make the path (if necessary)
-                make_path(sample.general.bestassembliespath)
                 # Set the name of the file in the best assemblies folder
                 bestassemblyfile = os.path.join(sample.general.bestassembliespath, '{}.fasta'.format(sample.name))
                 # Add the name and path of the best assembly file to the metadata
@@ -352,7 +357,7 @@ class QualiMap(object):
                 delattr(sample.depth, 'coverage')
                 delattr(sample.depth, 'length')
                 delattr(sample.depth, 'stddev')
-            except KeyError:
+            except AttributeError:
                 pass
 
     @staticmethod
@@ -374,14 +379,13 @@ class QualiMap(object):
         self.metadata = inputobject.runmetadata.samples
         self.start = inputobject.starttime
         self.cpus = inputobject.cpus
+        self.threadlock = Lock()
         try:
             self.threads = int(self.cpus / len(self.metadata)) if self.cpus / len(self.metadata) > 1 else 1
         except TypeError:
             self.threads = self.cpus
         self.logfile = inputobject.logfile
         self.path = inputobject.path
-        # Define /dev/null
-        # self.fnull = open(os.devnull, 'wb')
         self.samversion = get_version(['samtools']).decode('utf-8').split('\n')[2].split()[1]
         # Initialise queues
         self.mapqueue = Queue(maxsize=self.cpus)
@@ -398,13 +402,13 @@ if __name__ == '__main__':
         def associate(self):
             # Get the sequences in the sequences folder into a list. Note that they must have a file extension that
             # begins with .fa
-            self.strains = [fasta for fasta in sorted(glob('{}*.fa*'.format(self.assemblypath)))
+            self.strains = [fasta for fasta in sorted(glob(os.path.join(self.assemblypath, '*.fa*')))
                             if '.fastq' not in fasta]
             for strain in self.strains:
                 # Extract the name of the strain from the path and file extension
                 strainname = os.path.split(strain)[1].split('.')[0]
                 # Find the corresponding fastq files for each strain
-                fastq = sorted(glob('{}{}*fastq*'.format(self.fastqpath, strainname)))
+                fastq = sorted(glob(os.path.join(self.fastqpath, '{}*fastq*'.format(strainname))))
                 # Ensure that fastq files are present for each assembly
                 assert fastq, 'Cannot find fastq files for strain {}'.format(strainname)
                 # Create the object
@@ -413,8 +417,6 @@ if __name__ == '__main__':
                 metadata.name = strainname
                 # Create the .general attribute
                 metadata.general = GenObject()
-                # Set the .general.filteredfile file to be the name and path of the sequence file
-                # metadata.general.filteredfile = strain
                 # Set the path of the assembly file
                 metadata.general.bestassembliespath = self.assemblypath
                 # Populate the .fastqfiles category of :self.metadata
@@ -476,4 +478,3 @@ if __name__ == '__main__':
     from time import time
     starttime = time()
     MetadataInit(starttime)
-    printtime('Assembly and characterisation complete', starttime)
